function [ scores ] = csa_analyze( csafile, outfile, startp, endp )
%CSA_ANALYZE csafile outfile startp endp - perform cleavage site analysis on csafile
%   csafile is a matlab .mat file with two variables - sequence and specmat
%   sequence - text string of protein under analysis
%   specmat - cell matrix of specificity values for the enzyme
%   preference for substrate
%   matrix format:  column 1: amino acid letter code
%                   columns 2-x: specificity numbers for substrate sites

if ~exist(csafile,'file')
    error('Input csafile does not exist.')
end

load(csafile);

if ~exist('sequence','var') || ~exist('specmat','var')
    error('Input csafile is not valid.');
end

filehdl = fopen(strcat(outfile, '.txt') ,'w');

sequence = upper(sequence);

% Export protein sequence

fprintf(filehdl,'PROTEIN SEQUENCE\r\n==================\r\n\r\n');

residuesperline = 60;
residuespergroup = 10; % Must be factor of residuesperline

seqlen = length(sequence);
numlines = ceil(seqlen / residuesperline);

for i = 1:numlines
    startpos = (i-1)*residuesperline + 1;
    
    endpos = 0;
    
    if i == numlines
        remainder = mod(seqlen,residuesperline);
        if remainder == 0
            remainder = residuesperline;
        end
        endpos = startpos + remainder - 1;
    else
        endpos = i*residuesperline;
    end
    
    curline = sequence(startpos:endpos);
    
    outline = '';
    curlinelen = length(curline);
    numgroups = ceil(curlinelen / residuespergroup);
    for j = 1:numgroups;
        linestartpos = (j-1)*residuespergroup + 1;
        lineendpos = 0;
        
        if j == numgroups
            remainder = mod(curlinelen,residuespergroup);
            if remainder == 0
                remainder = residuespergroup;
            end
            lineendpos = linestartpos + remainder - 1;
        else
            lineendpos = j*residuespergroup;
        end
        
        outline = [outline, curline(linestartpos:lineendpos), ' '];
    end
    
    fprintf(filehdl,'%i\t%s\t%i\r\n', startpos,outline,endpos);
end

fprintf(filehdl,'\r\nMolecular weight: %6.2f\r\n',molweight(sequence));

fprintf(filehdl,'\r\nCLEAVAGE SITE ANALYSIS\r\n=======================\r\n\r\n');

fprintf(filehdl,'Using site pockets %i through %i for analysis...\r\n\r\n', startp,endp);

scores = struct('sequence',{},'score',{}, ...
    'scoreNorm', {}, 'startpos',{},'endpos',{}, ...
    'eachScore', {}, 'rangeScores', {}, 'medianScore', {}, ...
    'leftcleavagelen',{},'rightcleavagelen',{}); 

% pre-define matrix for individual scores


testlen = endp - startp + 1;

numsequences = seqlen - testlen + 1;
% AA = zeros(1,testlen);
% scores.eachScore = AA;


singleLetterAA = {'G';'P';'A';'V';'L';'I';'M';'F';'Y';'W';'S';'T';'C';'N';'Q';'D';'E';'K';'R';'H'}; 

% Specificity matrix where the columns correspond to each subsite from P4'
% to P4 and the rows correspond to each of the 20 amino acids. Scores are
% from MEROPS, which are generated based on pure counts (i.e. out of all
% the substrate cleavages known, X number are known to the Gly in the P4'
% position, etc.) 
specmat = specmat.data;

% Determines the maximum scores/count for each of the subsites P4' through
% P4
totalNumSubstrates = sum(specmat,1);

for i = 1:numsequences
    curseq = sequence(i:i + testlen-1);
    % Calculate scores
  
    curscore = 0;
    curseqscore = zeros(1, testlen);
    curseqPscore = zeros(1, testlen);
    
    for j = 1:length(curseq)
        correctedJ = startp - 1 + j;
        specmatrow = find(strcmp(singleLetterAA,curseq(j)));
        curseqscore(1,j) = specmat(specmatrow,correctedJ); 
        curseqPscore(1,j) = specmat(specmatrow,correctedJ)/totalNumSubstrates(correctedJ); 
        curscore = curscore + specmat(specmatrow,correctedJ);
    end
    
    % Calculating range and median scores
    maxScore = max(curseqscore);
    minScore = min(curseqscore);
    range = [minScore; maxScore];
    
    medianSco = median(curseqscore);

%% Normalized Cumulative Score
    
    % Normalize matrix scores to the total num substrates used to fill the
    % MEROPS Specificity Matrix 
    % Snormi = counti / total
    
    Snormi = 0; 
    for ii = 1:length(curseq)
        Snormi = Snormi + curseqPscore (1, ii); 
    end
    
%%  
    % convert each score matrix to a string
    EachScoreStr = mat2str(curseqscore);
    
    % Fill in structure array
    scores(i).sequence = curseq;
    scores(i).score = curscore;
    scores(i).scoreNorm = Snormi;
    scores(i).startpos = i;
    scores(i).endpos = i + testlen - 1;
    scores(i).eachScore = EachScoreStr;
    scores(i).medianScore = medianSco;
    scores(i).rangeScores = range;
    scores(i).leftcleavagelen = i + (testlen / 2) - 1;
    scores(i).rightcleavagelen = seqlen - (i + (testlen / 2) - 1);
    scores(i).leftcleavageweight = molweight(sequence(1:i + (testlen / 2) - 1)) / 1e3;
    scores(i).rightcleavageweight = molweight(sequence(i + (testlen / 2):end)) / 1e3;
    
end

fprintf(filehdl, 'Scores sorted by cumulative score \n');

[unused, order] = sort([scores(:).score],2,'descend');
scores = scores(order); 
 
cumScores = zeros(length(scores), 2);

for i = 1:length(scores)
    
    fprintf(filehdl, '%i, \t', i);
    fprintf(filehdl,'Score: %i,\t Norm: %5.2f,\t Start Pos: %i\t %s\t End Pos: %i\t Individual Scores: %s\t Range: [%5.2f,\t  %5.2f]\t Median: %5.2f\t (size,molweight): L: (%i,%6.2f)\tR: (%i,%6.2f)\r\n',...
        scores(i).score, scores(i).scoreNorm, ...
        scores(i).startpos,scores(i).sequence,scores(i).endpos,...
        scores(i).eachScore, ...
        scores(i).rangeScores(1), scores(i).rangeScores(2), scores(i).medianScore,...
        scores(i).leftcleavagelen,scores(i).leftcleavageweight,...
        scores(i).rightcleavagelen,scores(i).rightcleavageweight);
    cumScores(i, 1) = scores(i).score;
    cumScores(i, 2) = scores(i).scoreNorm;
    
end

if length(scores) > 8
y1 = quantile(cumScores(:,1),0.95);
fprintf(filehdl, 'Top 95th percentile of scores cutoff: %i\t \n', y1 );


figure
subplot(1,2,1);
h1 = histogram(cumScores(:,1))
title('Cumulative Likelihood Scores');
xlabel('Score');
ylabel('Frequency');
subplot(1,2,2);
h2 = histogram(cumScores(:,2))
title('Normalized Likelihood Scores');
xlabel('Normalized Score');
ylabel('Frequency');
end

fclose(filehdl);

fprintf('Done. Results have been saved to %s\n',outfile);

end

